Package: rle
Version: 0.10.0
Date: 2025-05-28
Title: Common Functions for Run-Length Encoded Vectors
Authors@R: person("Pavel N.", "Krivitsky", role=c("aut","cre"), email="pavel@statnet.org", comment=c(ORCID="0000-0002-9101-3362"))
Description: Common 'base' and 'stats' methods for 'rle' objects, aiming to make it possible to treat them transparently as vectors.
Depends: R (>= 3.5)
Imports: methods
Copyright: file inst/COPYRIGHT
BugReports: https://github.com/statnet/rle/issues
License: GPL-3
RoxygenNote: 7.3.2.9000
Encoding: UTF-8
Suggests: covr
NeedsCompilation: yes
Packaged: 2025-05-28 13:03:45 UTC; pavel
Author: Pavel N. Krivitsky [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9101-3362>)
Maintainer: Pavel N. Krivitsky <pavel@statnet.org>
Repository: CRAN
Date/Publication: 2025-05-28 14:10:02 UTC
Built: R 4.3.3; x86_64-apple-darwin20; 2025-05-28 16:40:32 UTC; unix
Archs: rle.so.dSYM
