# Generated by roxygen2: do not edit by hand

S3method("$",rle)
S3method("$<-",rle)
S3method("[",rle)
S3method("[<-",rle)
S3method("[[",rle)
S3method("[[<-",rle)
S3method(Math,rle)
S3method(Ops,rle)
S3method(Summary,rle)
S3method(as.rle,default)
S3method(as.rle,rle)
S3method(c,rle)
S3method(compress,rle)
S3method(index_to_run,rle)
S3method(is.na,rle)
S3method(length,rle)
S3method(mean,rle)
S3method(rep,rle)
S3method(str,rle)
export(as.rle)
export(compress)
export(index_to_run)
importFrom(methods,is)
useDynLib(rle)
